@@ -24,7 +24,6 @@
 #include <config.h>
 #include "gnome-vfs-mime-handlers.h"
 
-#include "eggdesktopentries.h"
 #include "gnome-vfs-application-registry.h"
 #include "gnome-vfs-mime-info.h"
 #include "gnome-vfs-mime-info-cache.h"
@@ -41,10 +40,29 @@
 
 #define GCONF_DEFAULT_VIEWER_EXEC_PATH   "/desktop/gnome/applications/component_viewer/exec"
 
+#define MIXED_API_WARNING "Cannot call %s with a GNOMEVFSMimeApplication structure "\
+			  "constructed by the deprecated application registry", \
+			  __FUNCTION__
+
+struct _GnomeVFSMimeApplicationPrivate
+{
+	char *desktop_file_path;
+	char *generic_name;
+	char *icon;
+	char *exec;
+	char *binary_name;
+	gboolean supports_uris;
+	gboolean startup_notification;
+	char *startup_wm_class;
+};
+
 extern GList * _gnome_vfs_configuration_get_methods_list (void);
 
-static GnomeVFSResult expand_parameters                          (gpointer                  action,
-								  GnomeVFSMimeActionType    type,
+static GnomeVFSResult expand_application_parameters              (const char               *exec,
+								  GList                   **uri_list,
+								  int                      *argc,
+								  char                   ***argv);
+static GnomeVFSResult expand_component_parameters                (gpointer                  action,
 								  GList                    *uris,
 								  int                      *argc,
 								  char                   ***argv);
@@ -163,73 +181,6 @@ gnome_vfs_mime_get_default_action (const char *mime_type)
 	return action;
 }
 
-static gboolean
-mime_application_supports_uri_scheme (GnomeVFSMimeApplication *application,
-				      const char *uri_scheme)
-{
-	/* The default supported uri scheme is "file" */
-	if (application->supported_uri_schemes == NULL
-	    && g_ascii_strcasecmp ((const char *) uri_scheme, "file") == 0) {
-		return TRUE;
-	}
-	return g_list_find_custom (application->supported_uri_schemes,
-				   uri_scheme,
-				   (GCompareFunc)g_ascii_strcasecmp) != NULL;
-}
-
-
-/* This should be public, but we're in an API freeze */
-GnomeVFSMimeApplication *
-gnome_vfs_mime_get_default_application_for_scheme (const char *mime_type,
-						   const char *uri_scheme)
-{
-	char *default_application_id;
-	GnomeVFSMimeApplication *default_application;
-
-	default_application = NULL;
-
-	/* First, try the default for the mime type */
-	default_application_id = gnome_vfs_mime_get_default_desktop_entry (mime_type);
-
-	if (default_application_id != NULL
-	    && default_application_id[0] != '\0') {
-		default_application =
-			gnome_vfs_mime_application_new_from_id (default_application_id);
-		g_free (default_application_id);
-	}
-
-	if (default_application != NULL &&
-	    !mime_application_supports_uri_scheme (default_application, uri_scheme)) {
-		gnome_vfs_mime_application_free (default_application);
-		default_application = NULL;
-	}
-	
-	if (default_application == NULL) {
-		GList *all_applications, *l;
-			
-		/* Failing that, try something from the complete list */
-		all_applications = gnome_vfs_mime_get_all_desktop_entries (mime_type);
-
-		for (l = all_applications; l != NULL; l = l->next) {
-			default_application = 
-				gnome_vfs_mime_application_new_from_id ((char *) l->data);
-			
-			if (default_application != NULL &&
-			    !mime_application_supports_uri_scheme (default_application, uri_scheme)) {
-				gnome_vfs_mime_application_free (default_application);
-				default_application = NULL;
-			}
-			if (default_application != NULL)
-				break;
-		}
-		g_list_foreach (all_applications, (GFunc) g_free, NULL);
-		g_list_free (all_applications);
-	}
-
-	return default_application;
-}
-
-
 /**
  * gnome_vfs_mime_get_default_application:
  * @mime_type: A const char * containing a mime type, e.g. "image/png"
@@ -250,7 +201,6 @@ gnome_vfs_mime_get_default_application (const char *mime_type)
 
 	/* First, try the default for the mime type */
 	default_application_id = gnome_vfs_mime_get_default_desktop_entry (mime_type);
-
 	if (default_application_id != NULL
 	    && default_application_id[0] != '\0') {
 		default_application =
@@ -1057,6 +1007,25 @@ gnome_vfs_mime_remove_from_all_applications (const char *mime_type,
 	return GNOME_VFS_ERROR_DEPRECATED_FUNCTION;
 }
 
+/**
+ * gnome_vfs_mime_application_equal:
+ * @application: a #GnomeVFSMimeApplication
+ * 
+ * Compare @app_a and @app_b.
+ * 
+ * Return value: %TRUE if @a and @b are equal, %FALSE otherwise.
+ *
+ * Since: 2.8
+ **/
+gboolean
+gnome_vfs_mime_application_equal (GnomeVFSMimeApplication *app_a,
+				  GnomeVFSMimeApplication *app_b)
+{
+	g_return_val_if_fail (app_a != NULL, FALSE);
+	g_return_val_if_fail (app_b != NULL, FALSE);
+
+	return (strcmp (app_a->id, app_b->id) == 0);
+}
 
 /**
  * gnome_vfs_mime_application_copy:
@@ -1083,6 +1052,16 @@ gnome_vfs_mime_application_copy (GnomeVFSMimeApplication *application)
 	result->expects_uris = application->expects_uris;
 	result->supported_uri_schemes = copy_str_list (application->supported_uri_schemes);
 	result->requires_terminal = application->requires_terminal;
+	
+	result->priv = g_new0 (GnomeVFSMimeApplicationPrivate, 1);
+	result->priv->desktop_file_path = g_strdup (application->priv->desktop_file_path);
+	result->priv->generic_name = g_strdup (application->priv->generic_name);
+	result->priv->icon = g_strdup (application->priv->icon);
+	result->priv->exec = g_strdup (application->priv->exec); 
+	result->priv->binary_name = g_strdup (application->priv->binary_name);
+	result->priv->supports_uris = application->priv->supports_uris;
+	result->priv->startup_notification = application->priv->startup_notification;
+	result->priv->startup_wm_class = g_strdup (application->priv->startup_wm_class);
 
 	return result;
 }
@@ -1098,6 +1077,18 @@ void
 gnome_vfs_mime_application_free (GnomeVFSMimeApplication *application) 
 {
 	if (application != NULL) {
+		GnomeVFSMimeApplicationPrivate *priv = application->priv;
+	
+		if (priv != NULL) {
+			g_free (priv->desktop_file_path);
+			g_free (priv->generic_name);
+			g_free (priv->icon);
+			g_free (priv->exec);
+			g_free (priv->binary_name);
+			g_free (priv->startup_wm_class);
+		}
+		g_free (priv);
+
 		g_free (application->name);
 		g_free (application->command);
 		g_list_foreach (application->supported_uri_schemes,
@@ -1175,114 +1166,7 @@ gnome_vfs_mime_component_list_free (GList *list)
 GnomeVFSMimeApplication *
 gnome_vfs_mime_application_new_from_id (const char *id)
 {
-	EggDesktopEntries *entries;
-	GError *entries_error;
-	GnomeVFSMimeApplication *application;
-	char *filename, *p;
-
-	application = NULL;
-	entries_error = NULL;
-
-	filename = g_build_filename ("applications", id, NULL);
-
-	entries =
-		egg_desktop_entries_new_from_file (NULL,
-				EGG_DESKTOP_ENTRIES_GENERATE_LOOKUP_MAP |
-			        EGG_DESKTOP_ENTRIES_DISCARD_COMMENTS,
-				filename,
-				NULL);
-	g_free (filename);
-	
-	if (entries == NULL)
-		return NULL;
-
-	application = g_new0 (GnomeVFSMimeApplication, 1);
-
-	application->id = g_strdup (id);
-	application->name = egg_desktop_entries_get_locale_string (entries,
-								   egg_desktop_entries_get_start_group (entries),
-								   "Name", NULL, NULL);
-	if (application->name == NULL) 
-		goto error;
-
-	application->command = egg_desktop_entries_get_string (entries,
-			                                       egg_desktop_entries_get_start_group (entries),
-							       "Exec", NULL);
-
-	if (application->command == NULL) 
-		goto error;
-
-	application->requires_terminal = egg_desktop_entries_get_boolean (entries,
-			                                       egg_desktop_entries_get_start_group (entries),
-							       "Terminal", &entries_error);
-
-	if (entries_error != NULL) {
-		g_error_free (entries_error);
-                application->requires_terminal = FALSE;
-	}
-
-	egg_desktop_entries_free (entries);
-	entries = NULL;
-
-	/* Guess on these last fields based on parameters passed to Exec line
-	 */
-	if ((p = strstr (application->command, "%f")) != NULL
-		|| (p = strstr (application->command, "%d")) != NULL
-		|| (p = strstr (application->command, "%n")) != NULL) {
-
-	  	do {
-			*p = '\0';
-			p--;
-		} while (p >= application->command && g_ascii_isspace (*p)); 
-		
-		application->can_open_multiple_files = FALSE;
-		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS; 
-		application->supported_uri_schemes = NULL;
-	} else if ((p = strstr (application->command, "%F")) != NULL
-		   || (p = strstr (application->command, "%D")) != NULL
-		   || (p = strstr (application->command, "%N")) != NULL) {
-	  	do {
-			*p = '\0';
-			p--;
-		} while (p >= application->command && g_ascii_isspace (*p)); 
-		
-		application->can_open_multiple_files = TRUE;
-		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS; 
-		application->supported_uri_schemes = NULL;
-	} else if ((p = strstr (application->command, "%u")) != NULL) {
-		do {
-			*p = '\0';
-			p--;
-		} while (p >= application->command && g_ascii_isspace (*p)); 
-
-		application->can_open_multiple_files = FALSE;
-		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS; 
-		application->supported_uri_schemes = _gnome_vfs_configuration_get_methods_list ();
-	} else if ((p = strstr (application->command, "%U")) != NULL) {
-		do {
-			*p = '\0';
-			p--;
-		} while (p >= application->command && g_ascii_isspace (*p)); 
-
-		application->can_open_multiple_files = TRUE;
-		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS; 
-		application->supported_uri_schemes = _gnome_vfs_configuration_get_methods_list ();
-	} else {
-		application->can_open_multiple_files = FALSE;
-		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS_FOR_NON_FILES; 
-		application->supported_uri_schemes = _gnome_vfs_configuration_get_methods_list ();
-	} 
-
-	return application;
-
-error:
-	if (entries) 
-		egg_desktop_entries_free (entries);
-
-	if (application) 
-		gnome_vfs_mime_application_free (application);
-
-	return NULL;
+	return gnome_vfs_mime_application_new_from_desktop_id (id);
 }
 
 /** 
@@ -1352,8 +1236,8 @@ gnome_vfs_mime_action_launch_with_env (GnomeVFSMimeAction *action,
 					 
 	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
 	
-		result = expand_parameters (action->action.component, action->action_type,
-					    uris, &argc, &argv);
+		result = expand_component_parameters (action->action.component,
+					              uris, &argc, &argv);
 					    
 		if (result != GNOME_VFS_OK) {
 			return result;
@@ -1428,34 +1312,16 @@ gnome_vfs_mime_application_launch_with_env (GnomeVFSMimeApplication *app,
                                             char                   **envp)
 {
 	GnomeVFSResult result;
-	GList *u;
-	char *scheme;
 	char **argv;
 	int argc;
 	
 	g_return_val_if_fail (app != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 	g_return_val_if_fail (uris != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 	
-	/* check that all uri schemes are supported */
-	if (app->supported_uri_schemes != NULL) {
-		for (u = uris; u != NULL; u = u->next) {
-
-			scheme = gnome_vfs_get_uri_scheme (u->data);
-		
-			if (!g_list_find_custom (app->supported_uri_schemes,
-						 scheme, (GCompareFunc) strcmp)) {
-				g_free (scheme);
-				return GNOME_VFS_ERROR_NOT_SUPPORTED;
-			}
-			
-			g_free (scheme);
-		}
-	}
-
 	while (uris != NULL) {
 		
-		result = expand_parameters (app, GNOME_VFS_MIME_ACTION_TYPE_APPLICATION,
-				            uris, &argc, &argv);
+		result = expand_application_parameters (app->priv->exec, &uris,
+							&argc, &argv);
 		
 		if (result != GNOME_VFS_OK) {
 			return result;
@@ -1481,66 +1347,253 @@ gnome_vfs_mime_application_launch_with_env (GnomeVFSMimeApplication *app,
 		}
 		
 		g_strfreev (argv);
-		uris = uris->next;
-		
-		if (app->can_open_multiple_files) {
-			break;
-		}
 	}
 	
 	return GNOME_VFS_OK;		
 }
 
+static char *
+expand_macro_single (char macro, const char *uri)
+{
+	char *result = NULL, *path;
+
+	switch (macro) {
+		case 'u':
+		case 'U':	
+			result = g_shell_quote (uri);
+			break;
+		case 'f':
+		case 'F':
+			path = gnome_vfs_get_local_path_from_uri (uri);
+			if (path) {
+				result = g_shell_quote (path);
+				g_free (path);
+			}
+			break;
+		case 'd':
+		case 'D':
+			path = gnome_vfs_get_local_path_from_uri (uri);
+			if (path) {
+				result = g_shell_quote (g_path_get_dirname (path));
+				g_free (path);
+			}
+			break;
+		case 'n':
+		case 'N':
+			path = gnome_vfs_get_local_path_from_uri (uri);
+			if (path) {
+				result = g_shell_quote (g_path_get_basename (path));
+				g_free (path);
+			}
+			break;
+	}
+
+	return result;
+}
+
+static void
+expand_macro (char macro, GString *exec, GList **uri_list)
+{
+	GList *uris = *uri_list;
+	char *expanded;
+
+	g_return_if_fail (uris != NULL);
+	g_return_if_fail (exec != NULL);
+
+	if (uris == NULL) {
+		return;
+	}
+
+	switch (macro) {
+		case 'u':
+		case 'f':
+		case 'd':
+		case 'n':
+			expanded = expand_macro_single (macro, uris->data);
+			if (expanded) {
+				g_string_append (exec, expanded);
+				g_free (expanded);
+			}
+			uris = uris->next;
+			break;
+		case 'U':	
+		case 'F':
+		case 'D':
+		case 'N':
+			while (uris) {
+				expanded = expand_macro_single (macro, uris->data);
+				if (expanded) {
+					g_string_append (exec, expanded);
+					g_free (expanded);
+				}
+
+				uris = uris->next;
+
+				if (uris != NULL && expanded) {
+					g_string_append_c (exec, ' ');
+				}
+			}
+			break;
+	}
+
+	*uri_list = uris;
+}
+
 static GnomeVFSResult
-expand_parameters (gpointer                 action,
-		   GnomeVFSMimeActionType   type,
-                   GList                   *uris,
-		   int                     *argc,
-		   char                  ***argv)		   
+expand_application_parameters (const char     *exec,
+			       GList         **uris,
+			       int            *argc,
+			       char         ***argv)		   
+{
+	GList *uri_list = *uris;
+	const char *p = exec;
+	GString *expanded_exec = g_string_new (NULL);
+
+	g_return_val_if_fail (p != NULL, GNOME_VFS_ERROR_PARSE);
+
+	while (*p) {
+		if (p[0] == '%' && p[1] != '\0') {
+			expand_macro (p[1], expanded_exec, uris);
+			p++;
+		} else {
+			g_string_append_c (expanded_exec, *p);
+		}
+
+		p++;
+	}
+
+	/* No substitutions */
+	if (uri_list == *uris) {
+		return GNOME_VFS_ERROR_PARSE;
+	}
+
+	if (!g_shell_parse_argv (expanded_exec->str, argc, argv, NULL)) {
+		return GNOME_VFS_ERROR_PARSE;
+	}
+
+	return GNOME_VFS_OK;
+}
+
+#ifdef TEXT_EXEC_MACRO_EXPANSION
+static void
+print_expansion_data (GList *uris, const char *exec)
+{
+	GList *l;
+	int i;
+
+	g_print ("Exec %s\n", exec);
+
+	for (l = uris, i = 0; l != NULL; l = l->next, i++)
+	{
+		g_print ("URI %d: %s\n", i, (char *)l->data);
+	}
+}
+
+static void
+print_macro_expansion (char **argv, GnomeVFSResult res)
+{
+	int i;
+
+	if (res != GNOME_VFS_OK) {
+		g_print ("Error\n");
+	} else {
+		for (i = 0; argv[i] != NULL; i++)
+		{
+			g_print ("Arg %d: %s\n", i, argv[i]);
+		}
+	}
+}
+
+static void
+test_exec_array (const char **execs, GList *uris)
+{
+	int argc, i;
+	char **argv;
+
+	for (i = 0; execs[i] != NULL; i++)
+	{
+		GList *l = uris;
+
+		print_expansion_data (uris, execs[i]);
+		while (l != NULL) {
+			GnomeVFSResult res;
+
+			res = expand_application_parameters
+					(execs[i], &l, &argc, &argv);
+			print_macro_expansion (argv, res);
+			g_strfreev (argv);
+		}
+
+		g_print ("---------------------\n");
+	}
+}
+
+void test_exec_macro_expansion (void);
+
+void
+test_exec_macro_expansion (void)
+{
+	GList *uris = NULL;
+	
+	const char *local[] = { "test --open-file=%f",
+				"test --open-files %F",
+				"test %d",
+				"test %D",
+				"test %n",
+				"test %N",
+				NULL };
+
+	const char *remote[] = { "test --open-uri=%u",
+				"test --open-uris %U",
+				"test %u",
+				"test %U",
+				NULL };
+
+	uris = g_list_append (uris, "file:///home/test/test1.txt");
+	uris = g_list_append (uris, "file:///home/test/test2.txt");
+	test_exec_array (local, uris);
+	
+	uris = g_list_append (uris, "http://www.test.org/test1.txt");
+	uris = g_list_append (uris, "http://www.test.org/test2.txt");
+	test_exec_array (remote, uris);
+
+	g_list_free (uris);
+}
+#endif
+
+static GnomeVFSResult
+expand_component_parameters (gpointer                 action,
+			     GList                   *uris,
+			     int                     *argc,
+			     char                  ***argv)		   
 {
-	GnomeVFSMimeApplication *app = NULL;
 	Bonobo_ServerInfo *server = NULL;
 	GConfClient *client;
-	char *path;
 	char *command = NULL;
 	char **c_argv, **r_argv;
 	int c_argc, max_r_argc;
 	int i, c;
 	gboolean added_arg;
 
-	/* figure out what command to parse */	
-	switch (type) {
-	case GNOME_VFS_MIME_ACTION_TYPE_APPLICATION:
-		app = (GnomeVFSMimeApplication *) action;
-		command = g_strdup (app->command);
-		break;
-		
-	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
-		if (!gconf_is_initialized ()) {
-			if (!gconf_init (0, NULL, NULL)) {
-				return GNOME_VFS_ERROR_INTERNAL;
-			}
-		}
-	
-		client = gconf_client_get_default ();
-		g_return_val_if_fail (client != NULL, GNOME_VFS_ERROR_INTERNAL);
-	
-		command = gconf_client_get_string (client, GCONF_DEFAULT_VIEWER_EXEC_PATH, NULL);
-		g_object_unref (client);
-		
-		if (command == NULL) {
-			g_warning ("No default component viewer set\n");
+	if (!gconf_is_initialized ()) {
+		if (!gconf_init (0, NULL, NULL)) {
 			return GNOME_VFS_ERROR_INTERNAL;
 		}
-		
-		server = (Bonobo_ServerInfo *) action;
-		
-		break;
-
-	default:
-		g_assert_not_reached ();
 	}
-
+	
+	client = gconf_client_get_default ();
+	g_return_val_if_fail (client != NULL, GNOME_VFS_ERROR_INTERNAL);
+	
+	command = gconf_client_get_string (client, GCONF_DEFAULT_VIEWER_EXEC_PATH, NULL);
+	g_object_unref (client);
+		
+	if (command == NULL) {
+		g_warning ("No default component viewer set\n");
+		return GNOME_VFS_ERROR_INTERNAL;
+	}
+		
+	server = (Bonobo_ServerInfo *) action;
+		
 	if (!g_shell_parse_argv (command,
 				 &c_argc,
 				 &c_argv,
@@ -1559,45 +1612,9 @@ expand_parameters (gpointer                 action,
 		/* replace %s with the uri parameters */
 		if (strcmp (c_argv[c], "%s") == 0) {
 			while (uris != NULL) {
-				if (app != NULL) {
-					switch (app->expects_uris) {
-					case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS:
-						r_argv[i] = g_strdup (uris->data);
-						break;
-	
-					case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS:
-						r_argv[i] = gnome_vfs_get_local_path_from_uri (uris->data);
-						if (r_argv[i] == NULL) {
-							g_strfreev (c_argv);
-							g_strfreev (r_argv);
-							return GNOME_VFS_ERROR_NOT_SUPPORTED;
-						}
-						break;
-			
-					case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS_FOR_NON_FILES:
-						/* this really means use URIs for non local files */
-						path = gnome_vfs_get_local_path_from_uri (uris->data);
-			
-						if (path != NULL) {
-							r_argv[i] = path;
-						} else {
-							r_argv[i] = g_strdup (uris->data);
-						}				
-			
-						break;
-				
-					default:
-						g_assert_not_reached ();
-					}	
-				} else {
-					r_argv[i] = g_strdup (uris->data);
-				}
+				r_argv[i] = g_strdup (uris->data);
 				i++;
 				uris = uris->next;
-				
-				if (app != NULL && !app->can_open_multiple_files) {
-					break;
-				}
 			}
 			added_arg = TRUE;
 			
@@ -1616,44 +1633,9 @@ expand_parameters (gpointer                 action,
 	/* if there is no %s or %c, append the parameters to the end */
 	if (!added_arg) {
 		while (uris != NULL) {
-			if (app != NULL) {
-				switch (app->expects_uris) {
-				case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS:
-					r_argv[i] = g_strdup (uris->data);
-					break;
-
-				case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS:
-					r_argv[i] = gnome_vfs_get_local_path_from_uri (uris->data);
-					if (r_argv[i] == NULL) {
-						g_strfreev (r_argv);
-						return GNOME_VFS_ERROR_NOT_SUPPORTED;
-					}
-					break;
-		
-				case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS_FOR_NON_FILES:
-					/* this really means use URIs for non local files */
-					path = gnome_vfs_get_local_path_from_uri (uris->data);
-		
-					if (path != NULL) {
-						r_argv[i] = path;
-					} else {
-						r_argv[i] = g_strdup (uris->data);
-					}				
-		
-					break;
-			
-				default:
-					g_assert_not_reached ();
-				}
-			} else {
-				r_argv[i] = g_strdup (uris->data);
-			}
+			r_argv[i] = g_strdup (uris->data);
 			i++;
 			uris = uris->next;
-			
-			if (app != NULL && !app->can_open_multiple_files) {
-				break;
-			}
 		}
 	}		
 	
@@ -1692,3 +1674,541 @@ copy_str_list (GList *string_list)
 				       }
 	return g_list_reverse (copy);
 }
+
+static void
+guess_deprecated_fields_from_exec (GnomeVFSMimeApplication *application)
+{
+	char *p;
+
+	application->command = g_strdup (application->priv->exec);
+
+	/* Guess on these last fields based on parameters passed to Exec line
+	 */
+	if ((p = strstr (application->command, "%f")) != NULL
+		|| (p = strstr (application->command, "%d")) != NULL
+		|| (p = strstr (application->command, "%n")) != NULL) {
+
+	  	do {
+			*p = '\0';
+			p--;
+		} while (p >= application->command && g_ascii_isspace (*p)); 
+		
+		application->can_open_multiple_files = FALSE;
+		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS; 
+		application->supported_uri_schemes = NULL;
+	} else if ((p = strstr (application->command, "%F")) != NULL
+		   || (p = strstr (application->command, "%D")) != NULL
+		   || (p = strstr (application->command, "%N")) != NULL) {
+	  	do {
+			*p = '\0';
+			p--;
+		} while (p >= application->command && g_ascii_isspace (*p)); 
+		
+		application->can_open_multiple_files = TRUE;
+		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS; 
+		application->supported_uri_schemes = NULL;
+	} else if ((p = strstr (application->command, "%u")) != NULL) {
+		do {
+			*p = '\0';
+			p--;
+		} while (p >= application->command && g_ascii_isspace (*p)); 
+
+		application->can_open_multiple_files = FALSE;
+		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS; 
+		application->supported_uri_schemes = _gnome_vfs_configuration_get_methods_list ();
+	} else if ((p = strstr (application->command, "%U")) != NULL) {
+		do {
+			*p = '\0';
+			p--;
+		} while (p >= application->command && g_ascii_isspace (*p)); 
+
+		application->can_open_multiple_files = TRUE;
+		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS; 
+		application->supported_uri_schemes = _gnome_vfs_configuration_get_methods_list ();
+	} else {
+		application->can_open_multiple_files = FALSE;
+		application->expects_uris = GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS_FOR_NON_FILES; 
+		application->supported_uri_schemes = _gnome_vfs_configuration_get_methods_list ();
+	} 
+}
+
+/**
+ * gnome_vfs_mime_application_new_from_desktop_id:
+ * @id: the identifier of a desktop entry
+ *
+ * Returns a new #GnomeVFSMimeApplication.
+ *
+ * Return value: a #GnomeVFSMimeApplication
+ *
+ * Since: 2.8
+ */
+GnomeVFSMimeApplication *
+gnome_vfs_mime_application_new_from_desktop_id (const char *id)
+{
+	GError *err = NULL;
+	GKeyFile *key_file;
+	GnomeVFSMimeApplication *app;
+	char **argv;
+	int argc;
+	char *filename;
+
+	g_return_val_if_fail (id != NULL, NULL);
+
+	app = g_new0 (GnomeVFSMimeApplication, 1);
+	app->priv = g_new0 (GnomeVFSMimeApplicationPrivate, 1);
+	app->id = g_strdup (id);
+
+	filename = g_build_filename ("applications", id, NULL);
+	key_file = g_key_file_new ();
+
+	if (!g_key_file_load_from_data_dirs (key_file, filename,
+					     &app->priv->desktop_file_path,
+					     G_KEY_FILE_NONE, &err)) {
+		goto exit;
+	}
+
+	app->name = g_key_file_get_locale_string (key_file, DESKTOP_ENTRY_GROUP,
+						  "Name", NULL, &err);
+	if (err) goto exit;
+
+	/* Not REQUIRED by the specification but required in our context */
+	app->priv->exec = g_key_file_get_string (key_file, DESKTOP_ENTRY_GROUP,
+						 "Exec", &err);
+	if (err) goto exit;
+
+	/* If there is no macro default to %f. This is also what KDE does */
+	if (!strchr (app->priv->exec, '%')) {
+		char *exec;
+
+		exec = g_strconcat (app->priv->exec, " %f", NULL);
+		g_free (app->priv->exec);
+		app->priv->exec = exec;
+	}
+
+	if (!g_shell_parse_argv (app->priv->exec, &argc, &argv, NULL)) {
+		goto exit;
+	}
+	app->priv->binary_name = g_strdup (argv[0]);
+	g_strfreev (argv);
+
+	app->requires_terminal = g_key_file_get_boolean
+			(key_file, DESKTOP_ENTRY_GROUP, "Terminal", &err);
+	if (err) {
+		g_error_free (err);
+		err = NULL;
+		app->requires_terminal = FALSE;
+	}
+
+	app->priv->startup_notification = g_key_file_get_boolean
+			(key_file, DESKTOP_ENTRY_GROUP, "StartupNotify", &err);
+	if (err) {
+		g_error_free (err);
+		err = NULL;
+		app->priv->startup_notification = FALSE;
+	}
+
+	app->priv->generic_name = g_key_file_get_locale_string
+			(key_file, DESKTOP_ENTRY_GROUP, "GenericName", NULL, NULL);
+ 
+	app->priv->icon = g_key_file_get_string
+			(key_file, DESKTOP_ENTRY_GROUP, "IconName", NULL);	
+	
+	app->priv->startup_wm_class = g_key_file_get_string
+			(key_file, DESKTOP_ENTRY_GROUP, "StartupWMClass", NULL);
+
+	app->priv->supports_uris = strstr (app->priv->exec, "%u") ||
+				   strstr (app->priv->exec, "%U");
+
+	guess_deprecated_fields_from_exec (app);
+
+exit:
+	g_free (filename);
+	g_key_file_free (key_file);
+
+	if (err) {
+		g_error_free (err);
+		gnome_vfs_mime_application_free (app);
+		return NULL;
+	}
+
+	return app;
+}
+
+/**
+ * gnome_vfs_mime_application_get_default_application_for_uri:
+ * @mime_type: a const char * containing a mime type, e.g. "application/x-php"
+ * @uri: a stringified URI
+ *
+ * Query the MIME database for the application to be executed on the file
+ * identified by @uri of MIME type @mime_type by default.
+ * 
+ * Return value: A GnomeVFSMimeApplication representing the default handler
+ *
+ * Since: 2.8
+ */
+GnomeVFSMimeApplication *
+gnome_vfs_mime_get_default_application_for_uri (const char *uri,
+						const char *mime_type)
+{
+	GList *applications, *l;
+	GnomeVFSMimeApplication *app = NULL;
+	char *scheme;
+	gboolean local;
+	
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (mime_type != NULL, NULL);
+	
+	scheme = gnome_vfs_get_uri_scheme (uri);
+	if (scheme != NULL) {
+		local = (strcmp (scheme, "file") == 0);
+	} else {
+		return NULL;
+	}
+	g_free (scheme);
+	
+	app = gnome_vfs_mime_get_default_application (mime_type);
+	if (local || gnome_vfs_mime_application_supports_uris (app)) {
+		return app;
+	} else {
+		gnome_vfs_mime_application_free (app);
+		app = NULL;
+	}
+
+	applications = gnome_vfs_mime_get_all_desktop_entries (mime_type);
+
+	for (l = applications; l != NULL; l = l->next) {
+		app = gnome_vfs_mime_application_new_from_id (l->data);
+
+		if (app != NULL)
+		{
+			if (local || gnome_vfs_mime_application_supports_uris (app)) {
+				break;
+			} else {
+				gnome_vfs_mime_application_free (app);
+				app = NULL;
+			}
+		}
+	}
+
+	g_list_foreach (applications, (GFunc) g_free, NULL);
+	g_list_free (applications);
+
+	return app;
+}
+
+/**
+ * gnome_vfs_mime_get_all_applications_for_uri:
+ * @mime_type: a const char * containing a mime type, e.g. "application/x-php"
+ * @uri: a stringified URI
+ * 
+ * Return an alphabetically sorted list of GnomeVFSMimeApplication
+ * data structures representing all applications in the MIME database able
+ * to handle the file identified by @uri of MIME type @mime_type (and supertypes).
+ * 
+ * Return value: A GList * where the elements are GnomeVFSMimeApplication *
+ * representing all possible handlers
+ *
+ * Since: 2.8
+ */
+GList *
+gnome_vfs_mime_get_all_applications_for_uri (const char *uri,
+					     const char *mime_type)
+{
+	GList *applications, *l, *result = NULL;
+	GnomeVFSMimeApplication *app;
+	char *scheme;
+	gboolean local;
+	
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (mime_type != NULL, NULL);
+	
+	scheme = gnome_vfs_get_uri_scheme (uri);
+	if (scheme != NULL) {
+		local = (strcmp (scheme, "file") == 0);
+	} else {
+		return NULL;
+	}
+	g_free (scheme);
+
+	applications = gnome_vfs_mime_get_all_desktop_entries (mime_type);
+
+	for (l = applications; l != NULL; l = l->next) {
+		app = gnome_vfs_mime_application_new_from_id (l->data);
+
+		if (app != NULL)
+		{
+			if (local || gnome_vfs_mime_application_supports_uris (app)) {
+				result = g_list_append (result, app);
+			} else {
+				gnome_vfs_mime_application_free (app);
+			}
+		}
+	}		
+
+	g_list_foreach (applications, (GFunc) g_free, NULL);
+	g_list_free (applications);
+
+	return result;
+}
+
+/**
+ * gnome_vfs_mime_application_get_desktop_file_path:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns the identifier of the desktop entry.
+ *
+ * Return value: the identifier of the desktop entry
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_desktop_id (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return NULL;
+	}
+
+	return app->id;
+}
+
+/**
+ * gnome_vfs_mime_application_get_desktop_file_path:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns the path of the desktop entry, a configuration
+ * file describing the properties of a particular program like
+ * it's name, how it is to be launched, how it appears in menus.
+ *
+ * Return value: the path of the .desktop file
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_desktop_file_path (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return NULL;
+	}
+
+	return app->priv->desktop_file_path;
+}
+
+/**
+ * gnome_vfs_mime_application_get_name:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns the name of the application.
+ *
+ * Return value: the name of the application.
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_name (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	return app->name;
+}
+
+/**
+ * gnome_vfs_mime_application_get_generic_name:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns the generic name of the application.
+ *
+ * Return value: the generic name of the application.
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_generic_name (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return NULL;
+	}
+
+	return app->priv->generic_name;
+}
+
+/**
+ * gnome_vfs_mime_application_get_icon:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns an icon representing the specified application.
+ *
+ * Return value: the filename of the icon usually without path information,
+ * e.g. "gedit-icon.png", and sometimes does not have an extension,
+ * e.g. "gnome-pdf" if the icon is supposed to be image type agnostic between
+ * icon themes. Icons are generic, and not theme specific.
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_icon (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return NULL;
+	}
+
+	return app->priv->icon;
+}
+
+/**
+ * gnome_vfs_mime_application_get_exec:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns the program to execute, possibly with arguments
+ * and parameter variables, as specified by the
+ * <ulink url="http://www.freedesktop.org/standards/desktop-entry-spec">
+ * Desktop Entry Specification</ulink>
+ *
+ * Return value: the command line to execute
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_exec (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return NULL;
+	}
+
+	return app->priv->exec;
+}
+
+/**
+ * gnome_vfs_mime_application_get_binary_name:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns the binary name of the specified application.
+ * Useful to implement startup notification.
+ * Note that this only provide partial information about
+ * application execution, it misses arguments and macros.
+ * DO NOT USE it to launch the application.
+ * Use gnome_vfs_mime_application_launch or
+ * gnome_vfs_mime_application_get_exec if you really
+ * need to write a custom launcher.
+ *
+ * Return value: the application's binary name
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_binary_name (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return NULL;
+	}
+
+	return app->priv->binary_name;
+}
+
+/**
+ * gnome_vfs_mime_application_supports_uris:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns whether the application accept uris as command
+ * lines arguments.
+ *
+ * Return value: %TRUE if the application can handle uris
+ *
+ * Since: 2.8
+ */
+gboolean
+gnome_vfs_mime_application_supports_uris (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, FALSE);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return FALSE;
+	}
+
+	return app->priv->supports_uris;
+}
+
+/**
+ * gnome_vfs_mime_application_requires_terminal:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns whether the application runs in a terminal window.
+ *
+ * Return value: %TRUE if the application runs in a terminal
+ *
+ * Since: 2.8
+ */
+gboolean
+gnome_vfs_mime_application_requires_terminal (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, FALSE);
+	
+	return app->requires_terminal;
+}
+
+/**
+ * gnome_vfs_mime_application_supports_startup_notification:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns whether the application supports startup notification.
+ * If true, it is KNOWN that the application will send a
+ * "remove" message when started with the DESKTOP_LAUNCH_ID
+ * environment variable set.
+ *
+ * Return value: %TRUE if the application supports startup notification
+ *
+ * Since: 2.8
+ */
+gboolean
+gnome_vfs_mime_application_supports_startup_notification (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, FALSE);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return FALSE;
+	}
+
+	return app->priv->startup_notification;
+}
+
+/**
+ * gnome_vfs_mime_application_get_startup_wm_class:
+ * @app: a #GnomeVFSMimeApplication
+ *
+ * Returns the WM class of the main window of the application.
+ *
+ * Return value: The WM class of the application's window
+ *
+ * Since: 2.8
+ */
+const char *
+gnome_vfs_mime_application_get_startup_wm_class (GnomeVFSMimeApplication *app)
+{
+	g_return_val_if_fail (app != NULL, NULL);
+	
+	if (app->priv == NULL) {
+		g_warning (MIXED_API_WARNING);
+		return NULL;
+	}
+
+	return app->priv->startup_wm_class;
+}
